--[[--------------------------------------------------------------------------------------------------------------

Angry Fish

Template object by Sergey "GrayFace" Rozhenko, based on 18:3 fish by Nifflas

Parameters:
1) Template index
2) Width of the trajectory (defaults to 50)
3) Height of the trajectory (defaults to 60)
4) Delay between jumps (defaults to 100)
5) Gravity (defaults to 45)

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 155, "This level requires a newer version of KS Ex")

local TemplateIndex, Width, Height, Delay, Accel = ...
Width, Height, Delay, Accel = Width or 50, Height or 60, Delay or 100, (Accel or 45)/200

local Path = path.dir(debug.FunctionFile(1))

ReplaceGraphics({
	{HotSpotX = 5, HotSpotY = 7, TransparentColor = -1},
	Path.."AngryFish_0.png",
	Path.."AngryFish_1.png",
	Path.."AngryFish_2.png",
	{HotSpotX = 6, HotSpotY = 7, TransparentColor = -1},
	Path.."AngryFish_10.png",
	Path.."AngryFish_11.png",
	Path.."AngryFish_12.png",
}, 254, assert(TemplateIndex))

AnimateObjects({
	right = {0, 2, 1, Loop = true, Delay = 2},
	left = {10, 12, 11, Loop = true, Delay = 2},
}, "right", 254, TemplateIndex)

-- Math:
-- SpeedY = Accel*T
-- H = SpeedY*T - Accel*T*T/2  = Accel*T*T/2
-- T = sqrt(2*H/Accel)
-- SpeedX = Width/(T*2)
-- t = [-T, T]
-- t1 = t + T
-- X = x0 + Width/2*t/T
-- Y = y0 - (Accel*T*t1 - Accel*t1*t1/2)  = y0 - (Accel*T*T - Accel*t*t/2 - Accel*T*T/2) = y0 - H + Accel*t*t/2
local T = math.sqrt(2*Height/Accel)
local Width2 = Width/2

events.global["NewTemplate"..TemplateIndex] = function(o)
	o.DoesHurt = 1
	o.DetectRed = 1
	local x, y = o:GetPosition()
	y = y + 18
	local t, dt = math.floor(-T), 1
	local InDelay = math.random(0, Delay)
	o:SetPosition(x - math.ceil(Width2), y)
	o:event("Timer", function()
		if InDelay > 0 then
			InDelay = InDelay - 1
		else
			t = t + dt
			o:SetAngle(t*dt > 0 and 180 or 0, false)
			o:SetPosition(x + Width2*t/T, y - Height + Accel*t*t/2)
			if math.abs(t) > T then
				dt = -dt
				InDelay = Delay
				o:Animate(dt > 0 and "right" or "left")
			end
		end
	end)
end
