--[[--------------------------------------------------------------------------------------------------------------

Collectables

Template object by Sergey "GrayFace" Rozhenko. Collectables graphics and idea by egomassive from KS+ mod.

Parameters:
1) Template index
2) Animation table
3) Power animation table
4) Collectable variable name
5) Show power on power bar only after the item is picked up
6) Allow only one instance of collectable. Makes collectable act like normal power.

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 157, "This level requires a newer version of KS Ex")

local TemplateIndex, ObjAnim, PowerAnim, Name, ShowPowerTemporary, OnlyOne = ...

local Path = path.dir(debug.FunctionFile(1))

ReplaceGraphics({BaseIndex = 1, Path.."Object", 1}, 254, assert(TemplateIndex))
if ObjAnim then
	AnimateObjects(nil, ObjAnim, 254, TemplateIndex)
end

local TakenStr = "_ObjectsRemoved"

-- TakenItems[PosStr] -> vars[Name] if OnlyOne
local function Taken(o)
	if OnlyOne then
		return vars, Name
	end
	return vars[TakenStr] or {}, ("%s_%s_%s"):format(Game.MapX - 1000, Game.MapY - 1000, o.BaseX + 25*(o.BaseY + 10*o.Layer))
end

local ObjCount = 0

local function update()
	while true do
		local o = Objects.PlayerPos:FindTemplateOverlap(TemplateIndex)
		if not o then
			return
		end
		local TakenItems, PosID = Taken(o)
		TakenItems[PosID] = 1
		if not OnlyOne then
			vars[Name] = vars[Name] + 1
			vars[TakenStr] = TakenItems
		end
		PickUpLight(o:GetPosition())
		o:Destroy()
		PlaySound(41)
	end
end

function events.global.DestroyObject(o)
	if o.Bank == 254 and o.Obj == TemplateIndex then
		ObjCount = ObjCount - 1
		if ObjCount == 0 then
			events.global.Remove("Timer", update)
		end
	end
end

events.global["NewTemplate"..TemplateIndex] = function(o)
	local TakenItems, PosID = Taken(o)
	if (TakenItems[PosID] or 0) > 0 then
		ObjCount = ObjCount + 2
		o:Destroy()
		ObjCount = ObjCount - 1
		return
	end

	if ObjCount == 0 then
		events.global.Timer = update
	end
	ObjCount = ObjCount + 1
end

-- Power in the bar
if not _G["_CollectableActivated_"..Name] then
	_G["_CollectableActivated_"..Name] = true

	function events.global.LoadGame()
		vars[Name] = vars[Name] or 0
	end

	local power, powerTpl
	local function UpdatePower()
		local need = not ShowPowerTemporary or vars[Name] ~= 0
		if need and not power then
			power = NewPower(Name, powerTpl, true)
			ReplaceGraphics({BaseIndex = 1, Path.."Power", 1}, power)
			power:Animate(PowerAnim)
		elseif not need and power then
			powerTpl = power.Obj
			RemovePower(power)
			power:Destroy()
			power = nil
		end
	end

	if ShowPowerTemporary then
		events.global.Timer = UpdatePower
	else
		UpdatePower()
	end
end
