--[[--------------------------------------------------------------------------------------------------------------

Lock for Collectables

Template object by Sergey "GrayFace" Rozhenko. Based on Nifflas' and egomassive's graphics.

Parameters:
1) Template index
2) Collectable variable name
3) Count needed to unlock (-N = subtract N from power count, 0 = like a key, +N = require N items)
4) Hide count required

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 155, "This level requires a newer version of KS Ex")

local TemplateIndex, Name, UnlockCount, HideCount = ...
UnlockCount = UnlockCount or 0
local UnlockModule = UnlockCount < 0 and -UnlockCount or math.max(1, UnlockCount)
HideCount = HideCount or UnlockCount == 0

local Path = path.dir(debug.FunctionFile(1))

ReplaceGraphics({Path.."Lock", 0, 8, TransparentColor = 0}, 254, assert(TemplateIndex))

local TakenStr = "_ObjectsRemoved"

local function PosStr(o)
	return UnlockCount < 0 and ("%s_%s_%s"):format(Game.MapX - 1000, Game.MapY - 1000, o.BaseX + 25*(o.BaseY + 10*o.Layer))
end

events.global["NewTemplate"..TemplateIndex] = function(o)
	local PosID = PosStr(o)
	if (vars[TakenStr] or {})[PosID] then
		return o:Destroy()
	end

	local CountTable = {}
	if not HideCount then
		local x, y = o:GetPosition()
		ShowPowerCount(UnlockCount, x + 10, y - 10, CountTable)
		for i, a in ipairs(CountTable) do
			a.Permanent = 0
			a:SetX(a:GetX() + i - 1)  -- compact them a bit
			a:SetLayer(1)
			a:MoveToBack()
			a:MoveAbove(o.ID)
			a:SetTransparency(20)
			a:Update()
		end
	end

	o.Solid = 1
	o.CanClimb = 1
	local function update()
		if UnlockCount < 0 then
			local solid = (vars[Name] or 0) < UnlockModule
			o.Solid = solid and 1 or 0
			o.CanClimb = solid and 1 or 0
			if solid or not o:CheckOverlap(Objects.PlayerPos) then
				return
			end
			vars[TakenStr] = vars[TakenStr] or {}
			vars[TakenStr][PosID] = 1
			vars[Name] = vars[Name] - UnlockModule
		elseif (vars[Name] or 0) < UnlockModule or GetDistance(o, Objects.PlayerPos) >= 40 then
			return
		end
		
		for i, a in ipairs(CountTable) do
			a:Destroy()
		end
		o:RemoveEvents()
		o.Solid = 0
		o.CanClimb = 0
		o:Animate{1, 8, Delay = 2, Next = function(self)
			return self:Destroy()
		end}
		PlaySound(42)
	end
	o:event("Timer", update)
	update()
end
