--[[--------------------------------------------------------------------------------------------------------------

Falling Platform

Template object by Sergey "GrayFace" Rozhenko

Parameters:
1) Template index
2) Delay, the opposite of speed at which the platform moves down (defaults to 2)
3) HotSpot X (defaults to 12)
4) HotSpot Y (defaults to 12)

--]]--------------------------------------------------------------------------------------------------------------

local TemplateIndex, Delay, HotX, HotY = ...
Delay = Delay or 2
HotX = HotX or 12
HotY = HotY or 12

local Path = path.dir(debug.FunctionFile(1))
ReplaceGraphics({Path.."Falling Platform.png", TransparentColor = 0xFF00FF,
	HotSpotX = HotX, HotSpotY = HotY}, 254, assert(TemplateIndex))

local function CreateList()
	RSObjFallingPlatforms = {}
	local platforms = RSObjFallingPlatforms
	function events.global.Timer()
		if next(platforms) then
			local pl = Objects.PlayerPos
			local y = pl:GetY()
			pl:SetY(y + 1)
			local t = {}
			for o in pairs(platforms) do
				if not o.ID then
					platforms[o] = nil
				elseif Objects.Player.Cheating == 0 and Objects.Platform.Gravity > 0 and pl:CheckOverlap(o) then
					o.ContactDelay = o.ContactDelay and o.ContactDelay - 1 or 0
					if o.ContactDelay <= 0 and not o:OverlapsBackdrop(0, 1) then
						o:SetY(o:GetY() + 1)
						o.Solid = 0
						t[o] = true
					end
				else
					o.ContactDelay = nil
				end
			end
			if next(t) and not pl:OverlapsObstacle() then
				for o in pairs(t) do
					o.ContactDelay = math.max(math.floor(o.ContactDelayBase), o.ContactDelay + o.ContactDelayBase)
					o.Solid = 1
					o:Update()
				end
				return pl:Update()
			end
			for o in pairs(t) do
				o:SetY(o:GetY() - 1)
				o.Solid = 1
			end
			pl:SetY(y)
		end
	end
end

events.global["NewTemplate"..TemplateIndex] = function(o)
	o.Solid = 1
	o.ContactDelayBase = Delay
	if BouncingBall then
		local mul = 1/(3*Delay)
		o.Deceleration = 45
		o.BounceMul = 0
		o:event("BallBounce", function(sx, sy)
			local pl = Objects.PlayerPos
			pl:SetY(pl:GetY() + 1)
			if sy > BouncingBall.SpeedY and Objects.PlayerPos:CheckOverlap(o) then
				o.SpeedY = o.SpeedY + (sy - BouncingBall.SpeedY + 200)*mul
			end
			pl:SetY(pl:GetY() - 1)
		end)
		o:event("Timer", function()
			ObjectMovement(o)
			ObjectBounce(o)
		end)
	else
		if not RSObjFallingPlatforms then
			CreateList()
		end
		RSObjFallingPlatforms[o] = true
	end
end
