--[[--------------------------------------------------------------------------------------------------------------

Block User

Template object by Sergey "GrayFace" Rozhenko. Idea and icon by egomassive from KS+.
Fully blocks player's regular controls.

Parameters:
1) Template index
2) Keep Down arrow reaction (for shifts)
3) Timeout after which the object releases player and possibly self-destructs
4) Keep it alive after the timeout?

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 157, "This level requires a newer version of KS Ex")

local TemplateIndex, KeepDown, Timeout, KeepAlive = ...
Timeout = Timeout or math.huge

local Path = path.dir(debug.FunctionFile(1))
ReplaceGraphics({TransparentColor = 0xFFFFFF, Path.."BlockUser.png"}, 254, assert(TemplateIndex))

local ObjCount = 0
local TimeLeft = Timeout

local function update()
	local o = Objects.PlayerPos:FindTemplateOverlap(TemplateIndex)
	if o then
		if TimeLeft == 0 then
			return KeepAlive or o:Destroy()
		end
		TimeLeft = TimeLeft - 1
		local k = Objects.KBD
		k.LeftRight = 0
		k.Jump = 0
		k.JumpOneshot = 0
		if not KeepDown or k.UpDown < 0 then
			k.UpDown = 0
		end
		if not KeepDown then
			k.DownOneshot = 0
		end
		k.Walk = 0
		k.ItemList = 0
		k.UmbrellaOneshot = 0
		k.Hologram = 0
	else
		TimeLeft = Timeout
	end
end

function events.global.DestroyObject(o)
	if o.Bank == 254 and o.Obj == TemplateIndex then
		ObjCount = ObjCount - 1
		if ObjCount == 0 then
			events.global.Remove("KeysFilter", update)
			TimeLeft = Timeout
		end
	end
end

events.global["NewTemplate"..TemplateIndex] = function(o)
	if ObjCount == 0 then
		events.global.KeysFilter = update
	end
	ObjCount = ObjCount + 1
	o:SetVisibility(false)
end
