--[[--------------------------------------------------------------------------------------------------------------

Golden Creature

Template object by Sergey "GrayFace" Rozhenko. Graphics and idea by egomassive from KS+ mod.

Parameters:
1) Template index
2) Power animation table
3) Collectable variable name ("GoldenCreatures")
4) Show power on power bar only after the item is picked up
5) Allow only one instance of the creature. This is the case in KS+ and is the standard.

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 157, "This level requires a newer version of KS Ex")

local TemplateIndex, PowerAnim, Name, ShowPowerTemporary, OnlyOne = ...
Name = Name or "GoldenCreatures"

local Path = path.dir(debug.FunctionFile(1))

local overlay, RealBank, RealObj = dofile(Path.."overlay.lua", Path)
local emit = dofile(Path.."emit.lua")

local CheckOverlap = emit.CheckOverlap or function(o, tpl)
	return o:CheckOverlap(Objects.PlayerPos)
end

local TakenStr = "_ObjectsRemoved"
local function PosStr(o)
	return OnlyOne and TemplateIndex or ("%s_%s_%s"):format(Game.MapX - 1000, Game.MapY - 1000, o.BaseX + 25*(o.BaseY + 10*o.Layer))
end

local function DoEmit(o)
	local c = Objects.Child(27)
	local x, y = o:GetPosition()
	if emit.RadX then
		x = x + math.random(-emit.RadX, emit.RadX)
	end
	c:SetPosition(x + emit.DX, y + emit.DY)
	c:MoveBelow(o.ID)
	c:SetDirection(math.random(emit.MinAngle, emit.MaxAngle))
end

events.global["NewTemplate"..TemplateIndex] = function(tpl)
	local PosID = PosStr(tpl)
	if (vars[TakenStr] or {})[PosID] then
		return tpl:Destroy()
	end
	local o = Objects.new(RealBank, RealObj, tpl.BaseX, tpl.BaseY, tpl.Layer)
	o.DoesHurt = 0
	tpl:SetVisibility(o:GetVisibility())
	overlay(tpl, o)
	o:SetVisibility(tpl:GetVisibility())
	o:SetTransparency(128)
	local pieces = {tpl, o}
	tpl.Pieces, o.Pieces = pieces, pieces

	tpl:event("DrawFrame", function()
		if not o.ID then
			return tpl:Destroy()
		end
		local vis = o:GetVisibility()
		tpl:SetVisibility(vis)
		if vis and not PlayerDead and CheckOverlap(o, tpl) then
			local TakenItems = vars[TakenStr] or {}
			if not TakenItems[PosID] then
				vars[Name] = vars[Name] + 1
			end
			TakenItems[PosID] = 1
			vars[TakenStr] = TakenItems
			local x, y = tpl:GetPosition()
			PickUpLight(x + (emit.PickDX or 0), y + (emit.PickDY or 0))
			o:Destroy()
			tpl:Destroy()
			PlaySound(41)
			if OnlyOne and emit.OnlyOne then
				for o in Objects.Enum{Bank = 254, Obj = TemplateIndex} do
					for _, o in ipairs(o.Pieces or {}) do
						o:Destroy()
					end
				end
			end
		else
			if emit.Period and math.random(emit.Period) == 1 then
				DoEmit(o)
			end
			if emit.Special and emit.Special(o, DoEmit) and math.random(emit.SpecialPeriod) == 1 then
				DoEmit(o)
			end
		end			
	end)
end

-- Power in the bar
if not _G["_CollectableActivated_"..Name] then
	_G["_CollectableActivated_"..Name] = true

	function events.global.LoadGame()
		vars[Name] = vars[Name] or 0
	end

	local power, powerTpl
	local function UpdatePower()
		local need = not ShowPowerTemporary or vars[Name] ~= 0
		if need and not power then
			power = NewPower(Name, powerTpl, true)
			ReplaceGraphics({BaseIndex = 1, Path.."Power", 1}, power)
			power:Animate(PowerAnim)
		elseif not need and power then
			powerTpl = power.Obj
			RemovePower(power)
			power:Destroy()
			power = nil
		end
	end

	if ShowPowerTemporary then
		events.global.Timer = UpdatePower
	else
		UpdatePower()
	end
end
