--[[--------------------------------------------------------------------------------------------------------------

Moving Platform

Template object by Sergey "GrayFace" Rozhenko

Parameters:
1) Template index
2) X speed (defaults to 100)
3) Y speed (defaults to 0)
4) Hotspot X (defaults to 12)
5) Hotspot Y (defaults to 12)
6) Jump-through offset (defaults to -6, set it to 'false' to make the platform fully solid)
7) Jump-through offset with inverted gravity (defaults to 5 + 5)
8) Kills player when smashed against a wall (defaults to 'false')

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 155, "This level requires a newer version of KS Ex")

local TemplateIndex, SpeedX, SpeedY, HotX, HotY, JumpThru, JumpThruI, Deadly = ...
SpeedX = SpeedX or 100
SpeedY = SpeedY or 0
HotX = HotX or 12
HotY = HotY or 12
if JumpThru ~= false then
	JumpThru = (JumpThru or -6) - HotY
	JumpThruI = (JumpThruI or 5 + 5) - HotY
end

local Path = path.dir(debug.FunctionFile(1))
ReplaceGraphics({Path.."Moving Platform.png", TransparentColor = 0xFF00FF,
	HotSpotX = HotX, HotSpotY = HotY}, 254, assert(TemplateIndex))
	
events.global["NewTemplate"..TemplateIndex] = function(o)
	local function IsSolid()
		local g = not BouncingBall and Objects.Platform.Gravity or BouncingBall.UpsideDown and -1 or 1
		if g >= 0 then
			return (Objects.PlayerPos:GetY() < o:GetY() + JumpThru) and 1 or 0
		else
			return (Objects.PlayerPos:GetY() > o:GetY() + JumpThruI + (BouncingBall and 0 or 3)) and 1 or 0
		end
	end
	o.Solid = JumpThru and IsSolid() or 1
	o.SpeedX = SpeedX
	o.SpeedY = SpeedY
	local moveX, moveY = 0, 0
	o:event("Timer", function()
		local solid = JumpThru and IsSolid() or 1
		o.Solid = solid
		moveX = moveX + o.SpeedX/100
		moveY = moveY + o.SpeedY/100
		if math.abs(moveX) < 1 and math.abs(moveY) < 1 then
			return
		end
		local deltaX = math[moveX > 0 and "floor" or "ceil"](moveX)
		local deltaY = math[moveY > 0 and "floor" or "ceil"](moveY)
		moveX = moveX - deltaX
		moveY = moveY - deltaY
		if not o:OverlapsBackdrop(deltaX, deltaY) then
			local pl = Objects.PlayerPos
			local x, y = pl:GetPosition()
			local function check(dx, dy)
				pl:SetPosition(x + dx, y + dy)
				return pl:CheckOverlap(o)
			end
			local collide = solid == 1 and Objects.Player.Cheating == 0 and check(-deltaX, -deltaY)
			if collide or solid == 1 and not BouncingBall and Objects.Player.Cheating == 0 and check(0, Objects.Platform.Gravity >= 0 and 1 or -1) then
				pl:SetPosition(x + deltaX, y + deltaY)
				o.Solid = 0
				if not pl:OverlapsObstacle() then
					o.Solid = 1
					return o:SetPosition(o:GetX() + deltaX, o:GetY() + deltaY)
				end
				o.Solid = 1
			end
			if not collide or Deadly then
				if collide then
					Die()
				end
				return pl:SetPosition(x, y), o:SetPosition(o:GetX() + deltaX, o:GetY() + deltaY)
			end
			pl:SetPosition(x, y)
		end
		o.SpeedX = -o.SpeedX
		o.SpeedY = -o.SpeedY
		--PlaySound(35)
	end)
	
	o:event("BounceSurfaceSpeed", function(t)
		if Objects.PlayerPos:CheckOverlap(o) then
			t.X = t.X + o.SpeedX
			t.Y = t.Y + o.SpeedY
		end
	end)
end
