--[[--------------------------------------------------------------------------------------------------------------

Touch Save Spot

Save spot objects triggered by touch by Sergey "GrayFace" Rozhenko, based on 0:1 Save Spot by Nifflas.
Made for VVVVVV and ball mode.

Parameters:
1) Template index
2) Is it an upside down save spot
3) Should Down arrow be ressed for it to work, like with standard save point

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 155, "This level requires a newer version of KS Ex")

local TemplateIndex, IsUp, NeedPress = ...
IsUp, NeedPress = not not IsUp, not not NeedPress

-- TouchSaveSpotLight global variable is used

local Path = path.dir(debug.FunctionFile(1))
ReplaceGraphics({Path.."Save Spot", 1, 9, BaseIndex = 1}, 254, assert(TemplateIndex))
AnimateObjects(nil, {0, 8, Loop = true, Delay = 4}, 254, TemplateIndex)

events.global["NewTemplate"..TemplateIndex] = function(o)
	o:SetEffect(1)
	if IsUp then
		o:SetAngle(180)
	end
	local SaveLight = {}
	local WasTouching
	if not NeedPress then
		local g = BouncingBall and BouncingBall.GravityY or Objects.Platform.Gravity
		local x, y = o:GetPosition()
		local x1, y1 = Objects.PlayerPos:GetPosition()
		WasTouching = (IsUp == (g < 0)) and (x == x1) and math.abs(y - y1) < 12
	end
	local function check()
		if SaveLight ~= TouchSaveSpotLight or not SaveLight.ID then
			local g = BouncingBall and BouncingBall.GravityY or Objects.Platform.Gravity
			local touch = (IsUp == (g < 0)) and Objects.PlayerPos:CheckOverlap(o) and
				(not NeedPress and IsKeysInputEnabled() or Objects.KBD.UpDown == 1)
			if touch and not WasTouching then
				SaveGame(o.BaseX, o.BaseY)
				SaveLight = Objects.Child(15)
				TouchSaveSpotLight = SaveLight
				SaveLight:SetAnimationDirection(IsUp and 16 or 0)
				SaveLight:MoveBelow(o.ID)
				local x, y = o:GetPosition()
				SaveLight:SetPosition(x - 12, y - 12)
				SaveLight:Destroy(true)
				PlaySound(46)
			end
			WasTouching = touch
		end
	end
	if BouncingBall then
		o:event("BallBounce", check)
		WasTouching = WasTouching and BouncingBall.SpeedX == 0 and BouncingBall.SpeedY == 0
	else
		o:event("Timer", check)
	end
end
