--[[--------------------------------------------------------------------------------------------------------------

Water Wheel

Template object by Sergey "GrayFace" Rozhenko

Parameters:
1) Template index
2) Rotation speed (defaults to 1 degree per tick)

--]]--------------------------------------------------------------------------------------------------------------

assert(Version >= 155, "This level requires a newer version of KS Ex")

local TemplateIndex, RotateSpeed = ...
RotateSpeed = RotateSpeed or 1

local Path = path.dir(debug.FunctionFile(1))

ReplaceGraphics({
	{HotSpotX = 38, HotSpotY = 38},
	Path.."WaterWheel.png",
}, 254, assert(TemplateIndex))

local abs = math.abs
local function sgn1(a)  -- not quite 'sgn'
	if a >= 0 then
		return 1
	else
		return -1
	end
end

local Ignore

events.global["NewTemplate"..TemplateIndex] = function(o)
	if not Ignore then
		Ignore = true
		local o1 = Objects.new(254, TemplateIndex, o.BaseX, o.BaseY, o.Layer)
		Ignore = nil
		local pieces = {o, o1}
		o.Pieces, o1.Pieces = pieces, pieces
		o.Solid, o1.Solid = 1, 1
		o:SetY(o:GetY() + 6)
		o1:SetPosition(o:GetPosition())
		o1:SetAngle(45)
		local angle = 0
		o:event("Timer", function()
			angle = angle + RotateSpeed
			if angle >= 360 then
				angle = angle - 360
			elseif angle < 0 then
				angle = angle + 360
			end
			o:SetAngle(angle)
			o:Update()
			o1:SetAngle(angle + 45)
			o1:Update()
			
			-- Here comes some math to push player in the right direction
			if not (Objects.PlayerPos:CheckOverlap(o) or Objects.PlayerPos:CheckOverlap(o1)) then  return  end
			local ox, oy = o:GetPosition()
			local x, y = Objects.PlayerPos:GetPosition()
			
			local nx, ny = x - ox, y - oy
			local tx, ty = nx + ny, ny - nx
			if RotateSpeed < 0 then
				tx, ty = -tx, -ty
			end
			local dx, dy = 0, 0
			local dx0, dy0
			for i = 1, 2 + (math.sqrt(nx*nx + ny*ny) + 7)*abs(RotateSpeed)*math.pi/180 do
				dx0, dy0 = dx, dy
				if abs(tx)*(abs(dy)+1) > abs(ty)*(abs(dx)+1) then
					dx = dx + sgn1(tx)
				else
					dy = dy + sgn1(ty)
				end
				Objects.PlayerPos:SetPosition(x + dx, y + dy)
				Objects.PlayerPos:Update()
				if not (Objects.PlayerPos:CheckOverlap(o) or Objects.PlayerPos:CheckOverlap(o1)) then
					dx0, dy0 = dx, dy
					break
				end
			end
			dx, dy = 0, 0
			while Objects.PlayerPos:OverlapsObstacle() and (nx + dx)*(nx + dx) + (ny + dy)*(ny + dy) < 50*50 do
				if abs(nx)*(abs(dy)+1) > abs(ny)*(abs(dx)+1) then
					dx = dx + sgn1(nx)
				else
					dy = dy + sgn1(ny)
				end
				Objects.PlayerPos:SetPosition(x + dx0 + dx, y + dy0 + dy)
				Objects.PlayerPos:Update()
				if not Objects.PlayerPos:OverlapsObstacle() then  return end
				Objects.PlayerPos:SetPosition(x + dx, y + dy)
				Objects.PlayerPos:Update()
			end
		end)
	end
end

